//
// DemoVectorR3Templates
//

// The purpose of this program is to introduce two programming tools:
//
//
//  1.  The VectorR3 class holds vectors in R^3 (3-space) with 
//			x, y, and z coordinates.  
//		It supports most of the operations you would wish to have.
//
//  2.  The C++ "template" feature can be used to write general routines
//			that work on a variety of classes.  This is illustrated by
//			writing a routine that computes the mean of a vector of
//			objects.  You do not need this routine for Math155B assignments,
//			it is for demonstration purposes only!


#include "../VrMath/LinearR3.h"		// Get definitions of the VectorR3 class

// The next line defines a function "CalcMean" (actually one such function
//	for each class T.  It takes a pointer to a list of objects of type T,
//  and an integer value, and returns an object of type.
template <class T> T CalcMean( T* arrayOfTs, long numObjects );

int main ()
{

	// A lot of simple operations work as you would expect on VectorR3 objects.
	// See the LinearR3.h file for even more.

	VectorR3 myFirstVec;				// Declares a vector 
	VectorR3 theSecondOne(1, 2.4, 3);	// Declares and initializes the x,y,z values of the vector

	myFirstVec.Set(3, 4, 5);			// Sets the entries in the vector
	myFirstVec = VectorR3(3, 4, 5);		// Another way to do the same thing.

	// Initialize an array of VectorR3 objects:
	VectorR3 anArrayOfVecs[3] = { VectorR3(1,2,3), VectorR3(4,5,6), VectorR3(-1,-2,-3) };

	myFirstVec = 2.0*theSecondOne;
	myFirstVec *= 2.0;
	myFirstVec = 0.5*myFirstVec - 0.5*theSecondOne;
	myFirstVec /= 3;								

	double dotProduct = (myFirstVec^theSecondOne);		// Compute the dot product.
		// Warning:  "^" is a very low priority operation, always use parens to be safe!

	VectorR3 theCrossProd = myFirstVec*theSecondOne;	// Compute the cross product.

	// Using a function defined by templates:

	double anArrayOfDoubles[3] = { 1, -1, 4 };

	double meanDbl = CalcMean ( anArrayOfDoubles, 3 );

	VectorR3 meanVec = CalcMean ( anArrayOfVecs, 3 );

	fprintf( stdout, "Mean of the reals is %f.\n", meanDbl );
	fprintf( stdout, "Mean of the vectors is (%f,%f,%f).\n", meanVec.x, meanVec.y, meanVec.z );

	fprintf (stdout, "\nPress enter to exit.\n" );
	char c = (char)fgetc ( stdin );
}

// Here the CalcMean routine is actually defined!
// It is required, but not checked, that numTs be positive.

template <class T> T CalcMean ( T* Tptr, long numTs )
{
	long i;
	T sum;
	sum = *(Tptr++);
	for (i=1; i<numTs; i++) {
		sum += *(Tptr++);
	}
	return sum/((double)numTs);
}
	
